@@ -7,43 +7,16 @@ use subs qq(dump);
 require Exporter;
 *import = \&Exporter::import;
 @EXPORT = qw(dd ddx);
-@EXPORT_OK = qw(dump pp quote);
+@EXPORT_OK = qw(dump pp dumpf quote);
 
-$VERSION = "1.14";
+$VERSION = "1.23";
 $DEBUG = 0;
 
 use overload ();
-use vars qw(%seen %refcnt @dump @fixup %require $TRY_BASE64);
+use vars qw(%seen %refcnt @dump @fixup %require $TRY_BASE64 @FILTERS $INDENT);
 
 $TRY_BASE64 = 50 unless defined $TRY_BASE64;
-
-my %is_perl_keyword = map { $_ => 1 }
-qw( __FILE__ __LINE__ __PACKAGE__ __DATA__ __END__ AUTOLOAD BEGIN CORE
-DESTROY END EQ GE GT INIT LE LT NE abs accept alarm and atan2 bind
-binmode bless caller chdir chmod chomp chop chown chr chroot close
-closedir cmp connect continue cos crypt dbmclose dbmopen defined
-delete die do dump each else elsif endgrent endhostent endnetent
-endprotoent endpwent endservent eof eq eval exec exists exit exp fcntl
-fileno flock for foreach fork format formline ge getc getgrent
-getgrgid getgrnam gethostbyaddr gethostbyname gethostent getlogin
-getnetbyaddr getnetbyname getnetent getpeername getpgrp getppid
-getpriority getprotobyname getprotobynumber getprotoent getpwent
-getpwnam getpwuid getservbyname getservbyport getservent getsockname
-getsockopt glob gmtime goto grep gt hex if index int ioctl join keys
-kill last lc lcfirst le length link listen local localtime lock log
-lstat lt m map mkdir msgctl msgget msgrcv msgsnd my ne next no not oct
-open opendir or ord pack package pipe pop pos print printf prototype
-push q qq qr quotemeta qw qx rand read readdir readline readlink
-readpipe recv redo ref rename require reset return reverse rewinddir
-rindex rmdir s scalar seek seekdir select semctl semget semop send
-setgrent sethostent setnetent setpgrp setpriority setprotoent setpwent
-setservent setsockopt shift shmctl shmget shmread shmwrite shutdown
-sin sleep socket socketpair sort splice split sprintf sqrt srand stat
-study sub substr symlink syscall sysopen sysread sysseek system
-syswrite tell telldir tie tied time times tr truncate uc ucfirst umask
-undef unless unlink unpack unshift untie until use utime values vec
-wait waitpid wantarray warn while write x xor y);
-
+$INDENT = "  " unless defined $INDENT;
 
 sub dump
 {
@@ -52,6 +25,8 @@ sub dump
     local %require;
     local @fixup;
 
+    require Data::Dump::FilterContext if @FILTERS;
+
     my $name = "a";
     my @dump;
 
@@ -92,13 +67,10 @@ sub dump
 
     if (%refcnt || %require) {
 	$out .= ";\n";
-	$out =~ s/^/  /gm;  # indent
+	$out =~ s/^/$INDENT/gm;
 	$out = "do {\n$out}";
     }
 
-    #use Data::Dumper;   print Dumper(\%refcnt);
-    #use Data::Dumper;   print Dumper(\%seen);
-
     print STDERR "$out\n" unless defined wantarray;
     $out;
 }
@@ -117,20 +89,31 @@ sub ddx {
     print $out;
 }
 
+sub dumpf {
+    require Data::Dump::Filtered;
+    goto &Data::Dump::Filtered::dump_filtered;
+}
+
 sub _dump
 {
     my $ref  = ref $_[0];
     my $rval = $ref ? $_[0] : \$_[0];
     shift;
 
-    my($name, $idx, $dont_remember) = @_;
+    my($name, $idx, $dont_remember, $pclass, $pidx) = @_;
 
     my($class, $type, $id);
-    if (overload::StrVal($rval) =~ /^(?:([^=]+)=)?([A-Z]+)\(0x([^\)]+)\)$/) {
-	$class = $1;
-	$type  = $2;
-	$id    = $3;
-    } else {
+    my $strval = overload::StrVal($rval);
+    # Parse $strval without using regexps, in order not to clobber $1, $2,...
+    if ((my $i = rindex($strval, "=")) >= 0) {
+	$class = substr($strval, 0, $i);
+	$strval = substr($strval, $i+1);
+    }
+    if ((my $i = index($strval, "(0x")) >= 0) {
+	$type = substr($strval, 0, $i);
+	$id = substr($strval, $i + 2, -1);
+    }
+    else {
 	die "Can't parse " . overload::StrVal($rval);
     }
     if ($] < 5.008 && $type eq "SCALAR") {
@@ -138,6 +121,50 @@ sub _dump
     }
     warn "\$$name(@$idx) $class $type $id ($ref)" if $DEBUG;
 
+    my $out;
+    my $comment;
+    my $hide_keys;
+    if (@FILTERS) {
+	my $pself = "";
+	$pself = fullname("self", [@$idx[$pidx..(@$idx - 1)]]) if $pclass;
+	my $ctx = Data::Dump::FilterContext->new($rval, $class, $type, $ref, $pclass, $pidx, $idx);
+	my @bless;
+	for my $filter (@FILTERS) {
+	    if (my $f = $filter->($ctx, $rval)) {
+		if (my $v = $f->{object}) {
+		    local @FILTERS;
+		    $out = _dump($v, $name, $idx, 1);
+		    $dont_remember++;
+		}
+		if (defined(my $c = $f->{bless})) {
+		    push(@bless, $c);
+		}
+		if (my $c = $f->{comment}) {
+		    $comment = $c;
+		}
+		if (defined(my $c = $f->{dump})) {
+		    $out = $c;
+		    $dont_remember++;
+		}
+		if (my $h = $f->{hide_keys}) {
+		    if (ref($h) eq "ARRAY") {
+			$hide_keys = sub {
+			    for my $k (@$h) {
+				return 1 if $k eq $_[0];
+			    }
+			    return 0;
+			};
+		    }
+		}
+	    }
+	}
+	push(@bless, "") if defined($out) && !@bless;
+	if (@bless) {
+	    $class = shift(@bless);
+	    warn "More than one filter callback tried to bless object" if @bless;
+	}
+    }
+
     unless ($dont_remember) {
 	if (my $s = $seen{$id}) {
 	    my($sname, $sidx) = @$s;
@@ -154,14 +181,21 @@ sub _dump
 	$seen{$id} = [$name, $idx];
     }
 
-    my $out;
-    if ($type eq "SCALAR" || $type eq "REF" || $type eq "REGEXP") {
+    if ($class) {
+	$pclass = $class;
+	$pidx = @$idx;
+    }
+
+    if (defined $out) {
+	# keep it
+    }
+    elsif ($type eq "SCALAR" || $type eq "REF" || $type eq "REGEXP") {
 	if ($ref) {
 	    if ($class && $class eq "Regexp") {
 		my $v = "$rval";
 
 		my $mod = "";
-		if ($v =~ /^\(\?([msix-]+):([\x00-\xFF]*)\)\z/) {
+		if ($v =~ /^\(\?\^?([msix-]*):([\x00-\xFF]*)\)\z/) {
 		    $mod = $1;
 		    $v = $2;
 		    $mod =~ s/-.*//;
@@ -188,14 +222,14 @@ sub _dump
 	    }
 	    else {
 		delete $seen{$id} if $type eq "SCALAR";  # will be seen again shortly
-		my $val = _dump($$rval, $name, [@$idx, "\$"]);
+		my $val = _dump($$rval, $name, [@$idx, "\$"], 0, $pclass, $pidx);
 		$out = $class ? "do{\\(my \$o = $val)}" : "\\$val";
 	    }
 	} else {
 	    if (!defined $$rval) {
 		$out = "undef";
 	    }
-	    elsif ($$rval =~ /^-?[1-9]\d{0,9}$/ || $$rval eq "0") {
+	    elsif (do {no warnings 'numeric'; $$rval + 0 eq $$rval}) {
 		$out = $$rval;
 	    }
 	    else {
@@ -213,7 +247,7 @@ sub _dump
     elsif ($type eq "GLOB") {
 	if ($ref) {
 	    delete $seen{$id};
-	    my $val = _dump($$rval, $name, [@$idx, "*"]);
+	    my $val = _dump($$rval, $name, [@$idx, "*"], 0, $pclass, $pidx);
 	    $out = "\\$val";
 	    if ($out =~ /^\\\*Symbol::/) {
 		$require{Symbol}++;
@@ -229,7 +263,7 @@ sub _dump
 		next if $k eq "SCALAR" && ! defined $$gval;  # always there
 		my $f = scalar @fixup;
 		push(@fixup, "RESERVED");  # overwritten after _dump() below
-		$gval = _dump($gval, $name, [@$idx, "*{$k}"]);
+		$gval = _dump($gval, $name, [@$idx, "*{$k}"], 0, $pclass, $pidx);
 		$refcnt{$name}++;
 		my $gname = fullname($name, $idx);
 		$fixup[$f] = "$gname = $gval";  #XXX indent $gval
@@ -241,7 +275,7 @@ sub _dump
 	my $tied = tied_str(tied(@$rval));
 	my $i = 0;
 	for my $v (@$rval) {
-	    push(@vals, _dump($v, $name, [@$idx, "[$i]"], $tied));
+	    push(@vals, _dump($v, $name, [@$idx, "[$i]"], $tied, $pclass, $pidx));
 	    $i++;
 	}
 	$out = "[" . format_list(1, $tied, @vals) . "]";
@@ -256,31 +290,38 @@ sub _dump
 	my $kstat_sum2 = 0;
 
 	my @orig_keys = keys %$rval;
+	if ($hide_keys) {
+	    @orig_keys = grep !$hide_keys->($_), @orig_keys;
+	}
 	my $text_keys = 0;
 	for (@orig_keys) {
-	    $text_keys++, last unless $_ eq "0" || /^[-+]?[1-9]\d*(?:.\d+)?\z/;
+	    $text_keys++, last unless /^[-+]?(?:0|[1-9]\d*)(?:\.\d+)?\z/;
 	}
 
 	if ($text_keys) {
-	    @orig_keys = sort @orig_keys;
+	    @orig_keys = sort { lc($a) cmp lc($b) } @orig_keys;
 	}
 	else {
 	    @orig_keys = sort { $a <=> $b } @orig_keys;
 	}
 
+	my $quote;
 	for my $key (@orig_keys) {
-	    my $val = \$rval->{$key};
-	    $key = quote($key) if $is_perl_keyword{$key} ||
-		                  !($key =~ /^[a-zA-Z_]\w{0,19}\z/ ||
-				    $key =~ /^-?[1-9]\d{0,8}\z/
-				    );
+	    next if $key =~ /^-?[a-zA-Z_]\w*\z/;
+	    next if $key =~ /^-?[1-9]\d{0,8}\z/;
+	    $quote++;
+	    last;
+	}
 
+	for my $key (@orig_keys) {
+	    my $val = \$rval->{$key};  # capture value before we modify $key
+	    $key = quote($key) if $quote;
 	    $kstat_max = length($key) if length($key) > $kstat_max;
 	    $kstat_sum += length($key);
 	    $kstat_sum2 += length($key)*length($key);
 
 	    push(@keys, $key);
-	    push(@vals, _dump($$val, $name, [@$idx, "{$key}"], $tied));
+	    push(@vals, _dump($$val, $name, [@$idx, "{$key}"], $tied, $pclass, $pidx));
 	}
 	my $nl = "";
 	my $klen_pad = 0;
@@ -310,20 +351,24 @@ sub _dump
 	    }
 	}
 	$out = "{$nl";
-	$out .= "  # $tied$nl" if $tied;
+	$out .= "$INDENT# $tied$nl" if $tied;
 	while (@keys) {
 	    my $key = shift @keys;
 	    my $val = shift @vals;
-	    my $pad = " " x ($klen_pad + 6);
-	    $val =~ s/\n/\n$pad/gm;
-	    $key = " $key" . " " x ($klen_pad - length($key)) if $nl;
-	    $out .= " $key => $val,$nl";
+	    my $vpad = $INDENT . (" " x ($klen_pad ? $klen_pad + 4 : 0));
+	    $val =~ s/\n/\n$vpad/gm;
+	    my $kpad = $nl ? $INDENT : " ";
+	    $key .= " " x ($klen_pad - length($key)) if $nl && $klen_pad > length($key);
+	    $out .= "$kpad$key => $val,$nl";
 	}
 	$out =~ s/,$/ / unless $nl;
 	$out .= "}";
     }
     elsif ($type eq "CODE") {
-	$out = 'sub { "???" }';
+	$out = 'sub { ... }';
+    }
+    elsif ($type eq "VSTRING") {
+        $out = sprintf +($ref ? '\v%vd' : 'v%vd'), $$rval;
     }
     else {
 	warn "Can't handle $type data";
@@ -333,6 +378,12 @@ sub _dump
     if ($class && $ref) {
 	$out = "bless($out, " . quote($class) . ")";
     }
+    if ($comment) {
+	$comment =~ s/^/# /gm;
+	$comment .= "\n" unless $comment =~ /\n\z/;
+	$comment =~ s/^#[ \t]+\n/\n/;
+	$out = "$comment$out";
+    }
     return $out;
 }
 
@@ -386,11 +437,39 @@ sub format_list
     my $paren = shift;
     my $comment = shift;
     my $indent_lim = $paren ? 0 : 1;
+    if (@_ > 3) {
+	# can we use range operator to shorten the list?
+	my $i = 0;
+	while ($i < @_) {
+	    my $j = $i + 1;
+	    my $v = $_[$i];
+	    while ($j < @_) {
+		# XXX allow string increment too?
+		if ($v eq "0" || $v =~ /^-?[1-9]\d{0,9}\z/) {
+		    $v++;
+		}
+		elsif ($v =~ /^"([A-Za-z]{1,3}\d*)"\z/) {
+		    $v = $1;
+		    $v++;
+		    $v = qq("$v");
+		}
+		else {
+		    last;
+		}
+		last if $_[$j] ne $v;
+		$j++;
+	    }
+	    if ($j - $i > 3) {
+		splice(@_, $i, $j - $i, "$_[$i] .. $_[$j-1]");
+	    }
+	    $i++;
+	}
+    }
     my $tmp = "@_";
     if ($comment || (@_ > $indent_lim && (length($tmp) > 60 || $tmp =~ /\n/))) {
 	my @elem = @_;
-	for (@elem) { s/^/  /gm; }   # indent
-	return "\n" . ($comment ? "  # $comment\n" : "") .
+	for (@elem) { s/^/$INDENT/gm; }
+	return "\n" . ($comment ? "$INDENT# $comment\n" : "") .
                join(",\n", @elem, "");
     } else {
 	return join(", ", @_);
@@ -402,7 +481,7 @@ sub str {
       for ($_[0]) {
       # Check for repeated string
       if (/^(.)\1\1\1/s) {
-          # seems to be a repating sequence, let's check if it really is
+          # seems to be a repeating sequence, let's check if it really is
           # without backtracking
           unless (/[^\Q$1\E]/) {
               my $base = quote($1);
@@ -428,6 +507,7 @@ sub str {
       # 17 bytes (not counting any require statement needed).
       # But on the other hand, hex is much more readable.
       if ($TRY_BASE64 && length($_[0]) > $TRY_BASE64 &&
+	  (defined &utf8::is_utf8 && !utf8::is_utf8($_[0])) &&
 	  eval { require MIME::Base64 })
       {
 	  $require{"MIME::Base64"}++;
@@ -479,17 +559,17 @@ Data::Dump - Pretty printing of data structures
 
 =head1 SYNOPSIS
 
- use Data::Dump qw(dump ddx);
+ use Data::Dump qw(dump);
 
  $str = dump(@list);
  @copy_of_list = eval $str;
 
  # or use it for easy debug printout
- ddx localtime;
+ use Data::Dump; dd localtime;
 
 =head1 DESCRIPTION
 
-This module provide functions that takes a list of values as their
+This module provide a few functions that traverse their
 argument and produces a string as its result.  The string contains
 Perl code that, when C<eval>ed, produces a deep copy of the original
 arguments.
@@ -502,7 +582,7 @@ that is easy to read.  Example:
 
 Produces:
 
-    (1, [2, 3], { 4 => 5 })
+    "(1, [2, 3], { 4 => 5 })"
 
 If you dump just a little data, it is output on a single line. If
 you dump data that is more complex or there is a lot of it, line breaks
@@ -548,7 +628,8 @@ Returns a quoted version of the provided string.
 
 It differs from C<dump($string)> in that it will quote even numbers and
 not try to come up with clever expressions that might shorten the
-output.
+output.  If a non-scalar argument is provided then it's just stringified
+instead of traversed.
 
 =item dd( ... )
 
@@ -563,13 +644,43 @@ it prints with "# " and mark the first line with the file and line
 number where it was called.  This is meant to be useful for debug
 printouts of state within programs.
 
+=item dumpf( ..., \&filter )
+
+Short hand for calling the dump_filtered() function of L<Data::Dump::Filtered>.
+This works like dump(), but the last argument should be a filter callback
+function.  As objects are visited the filter callback is invoked and it
+can modify how the objects are dumped.
+
+=back
+
+=head1 CONFIGURATION
+
+There are a few global variables that can be set to modify the output
+generated by the dump functions.  It's wise to localize the setting of
+these.
+
+=over
+
+=item $Data::Dump::INDENT
+
+This holds the string that's used for indenting multiline data structures.
+It's default value is "  " (two spaces).  Set it to "" to suppress indentation.
+Setting it to "| " makes for nice visuals even if the dump output then fails to
+be valid Perl.
+
+=item $Data::Dump::TRY_BASE64
+
+How long must a binary string be before we try to use the base64 encoding
+for the dump output.  The default is 50.  Set it to 0 to disable base64 dumps.
+
 =back
 
 
 =head1 LIMITATIONS
 
-Code references will be displayed as simply 'sub { "???" }' when
-dumped. Thus, C<eval>ing them will not reproduce the original routine.
+Code references will be dumped as C<< sub { ... } >>. Thus, C<eval>ing them will
+not reproduce the original routine.  The C<...>-operator used will also require
+perl-5.12 or better to be evaled.
 
 If you forget to explicitly import the C<dump> function, your code will
 core dump. That's because you just called the builtin C<dump> function
@@ -583,23 +694,24 @@ in-most-cases-excellent C<Data::Dumper>.  Basic ideas and some code
 are shared with Sarathy's module.
 
 The C<Data::Dump> module provides a much simpler interface than
-C<Data::Dumper>.  No OO interface is available and there are no
-configuration options to worry about (yet :-).  The other benefit is
+C<Data::Dumper>.  No OO interface is available and there are fewer
+configuration options to worry about.  The other benefit is
 that the dump produced does not try to set any variables.  It only
 returns what is needed to produce a copy of the arguments.  This means
-that C<dump("foo")> simply returns C<"foo">, and C<dump(1..5)> simply
-returns C<(1, 2, 3, 4, 5)>.
+that C<dump("foo")> simply returns C<'"foo"'>, and C<dump(1..3)> simply
+returns C<'(1, 2, 3)'>.
 
 =head1 SEE ALSO
 
-L<Data::Dumper>, L<Storable>
+L<Data::Dump::Filtered>, L<Data::Dump::Trace>, L<Data::Dumper>, L<JSON>,
+L<Storable>
 
 =head1 AUTHORS
 
 The C<Data::Dump> module is written by Gisle Aas <gisle@aas.no>, based
 on C<Data::Dumper> by Gurusamy Sarathy <gsar@umich.edu>.
 
- Copyright 1998-2000,2003-2004,2008 Gisle Aas.
+ Copyright 1998-2010 Gisle Aas.
  Copyright 1996-1998 Gurusamy Sarathy.
 
 This library is free software; you can redistribute it and/or
